@@ -9,17 +9,54 @@ from attrdict import AttrDict
 import attrdict
 from getpass import getpass
 
-__all__ = ['botlog', 'randomize_between', 'Bot']
-
-def botlog(message, error = False, _copy = False):
-    ERROR = "\033[1;31;40m"
-    NORMAL = "\033[m"
-    print(f'{ERROR if error else NORMAL}{"COPY: " if _copy else ""}{message}{NORMAL}')
-    sys.stdout.flush()
-
-randomize_between = lambda *strings : random.choice(strings)
-
 class Bot:
+
+    @staticmethod
+    def log(message, error = False, _copy = False):
+        ERROR = "\033[1;31;40m"
+        NORMAL = "\033[m"
+        print(f'{ERROR if error else NORMAL}{"COPY: " if _copy else ""}{message}{NORMAL}')
+        sys.stdout.flush()
+
+    @staticmethod
+    def randomize_between(*responses):
+        response = random.choice(responses)
+        if callable(response):
+            return response(self, msg, re.search(regex, msg.data.content))
+            if isinstance(result, str):
+                return result
+            elif isinstance(result, dict):
+                for send, nick in result.items():
+                    self.set_nick(nick)
+                    self.send_msg(send, parent = msg)
+                self.set_nick(self.nick)
+            elif isinstance(result, list):
+                for send in result:
+                    self.send_msg(send, parent = msg)
+        else:
+            if isinstance(response, str):
+                self.send_msg(msgString = result, parent = msg)
+            elif isinstance(response, dict):
+                for send, nick in response.items():
+                    self.set_nick(nick)
+                    self.send_msg(send, parent = msg)
+                self.set_nick(self.nick)
+            elif isinstance(response, list):
+                for send in response:
+                    self.send_msg(send, parent = msg)
+            else:
+                raise BadMessageError(f"Response type: {type(response)}")
+        break
+
+    @staticmethod
+    def add_command_line_arg(self, *args, **kwargs):
+        self.parser.add_argument(*args, **kwargs)
+        self.args = self.parser.parse_args()
+        if "name" in kwargs:
+            return self.args.kwargs["name"]
+        else:
+            return self.args
+
     def __init__(self, nick, **kwargs):
         vars = ["userEmail",
             "userPassword",
@@ -74,15 +111,7 @@ class Bot:
             self.room = self.args.room if self.args.test != True or _copy else "test"
             self.password = self.args.password
             if not self._copy:
-                botlog(f"Debug: {str(self.args.test)}", _copy = self._copy)
-
-    def add_command_line_arg(self, *args, **kwargs):
-        self.parser.add_argument(*args, **kwargs)
-        self.args = self.parser.parse_args()
-        if "name" in kwargs:
-            return self.args.kwargs["name"]
-        else:
-            return self.args
+                log(f"Debug: {str(self.args.test)}", _copy = self._copy)
 
     def connect(self):
         self.conn = websocket.create_connection(f'wss://euphoria.io/room/{self.room}/ws{"?h=0" if not self.human else "?h=1"}', cookie = self.cookie, enable_multithread = True)
@@ -98,8 +127,8 @@ class Bot:
             self._handle_auth(self.password)
             self.set_nick(self.nick)
         else:
-            botlog(reply, error = True, _copy = self._copy)
-        botlog(f'connected to &{self.room}.', _copy = self._copy)
+            log(reply, error = True, _copy = self._copy)
+        log(f'connected to &{self.room}.', _copy = self._copy)
         return hello
 
     def send_msg(self, msgString, parent = None):
@@ -120,14 +149,14 @@ class Bot:
                 if msg.type == "send-reply":
                     reply = msg
                     if isinstance(parent, AttrDict):
-                        botlog(f'Message sent: {reply} replying to: {parent.data.id} by {parent.data.sender.name}', _copy = self._copy)
+                        log(f'Message sent: {reply} replying to: {parent.data.id} by {parent.data.sender.name}', _copy = self._copy)
                     elif isinstance(parent, str):
-                        botlog(f'Message sent: {reply} replying to: {parent}', _copy = self._copy)
+                        log(f'Message sent: {reply} replying to: {parent}', _copy = self._copy)
                     elif parent is None:
-                        botlog(f'Message sent: {reply}', _copy = self._copy)
+                        log(f'Message sent: {reply}', _copy = self._copy)
                     return reply
                 if i > self.APITimeout:
-                    botlog("send-reply API response not recorded. reason: too many events before send-reply.", True, _copy = self._copy)
+                    log("send-reply API response not recorded. reason: too many events before send-reply.", True, _copy = self._copy)
                     return None
         except BadMessageError:
             pass
@@ -136,9 +165,9 @@ class Bot:
         if self.conn is not None:
             self.conn.close()
             del self.conn
-            botlog("restarting...", _copy = self._copy)
+            log("restarting...", _copy = self._copy)
         else:
-            botlog("starting...", _copy = self._copy)
+            log("starting...", _copy = self._copy)
         self.connect()
         self.start()
 
@@ -152,7 +181,7 @@ class Bot:
                     elif msg.type == 'send-event' and msg.data.sender.name != self.nick:
                         self._handle_message(msg)
                     elif msg.type == 'error':
-                        botlog(msg, error = True, _copy = self._copy)
+                        log(msg, error = True, _copy = self._copy)
                     elif msg.type == 'bounce-event':
                         self._handle_auth(msg)
                     else:
@@ -163,7 +192,7 @@ class Bot:
             if self.function is None and function is not None and callable(function):
                 self.function = function
             else:
-                botlog("No function passed to advanced bot's start function. Using filler function", _copy = self._copy)
+                log("No function passed to advanced bot's start function. Using filler function", _copy = self._copy)
                 self.function = lambda msg: None
             try:
                 for msgJSON in self.conn:
@@ -183,7 +212,7 @@ class Bot:
                         elif callable(self.function):
                             self.function(msg)
                         else:
-                            botlog("Advanced start must be given a callable message handler function that takes an AttrDict as its argument.", error = True, _copy = self._copy)
+                            log("Advanced start must be given a callable message handler function that takes an AttrDict as its argument.", error = True, _copy = self._copy)
             except KilledError:
                 pass
 
@@ -233,18 +262,29 @@ class Bot:
             if re.search(regex, msg.data.content) is not None:
                 if callable(response):
                     result = response(self, msg, re.search(regex, msg.data.content))
-                    if type(result) == str:
+                    if isinstance(result, str):
                         self.send_msg(msgString = result, parent = msg)
-                    elif type(result) == dict:
+                    elif isinstance(result, dict):
                         for send, nick in result.items():
                             self.set_nick(nick)
                             self.send_msg(send, parent = msg)
                         self.set_nick(self.nick)
-                    elif type(result) == list:
+                    elif isinstance(result, list):
                         for send in result:
                             self.send_msg(send, parent = msg)
                 else:
-                    self.send_msg(response, parent = msg)
+                    if isinstance(response, str):
+                        self.send_msg(msgString = result, parent = msg)
+                    elif isinstance(response, dict):
+                        for send, nick in response.items():
+                            self.set_nick(nick)
+                            self.send_msg(send, parent = msg)
+                        self.set_nick(self.nick)
+                    elif isinstance(response, list):
+                        for send in response:
+                            self.send_msg(send, parent = msg)
+                    else:
+                        raise BadMessageError(f"Response type: {type(response)}")
                 break
 
     def _handle_kill(self, msg):
@@ -269,13 +309,13 @@ class Bot:
             if msg.type == "auth-reply":
                 reply = msg
                 if reply.data.success == True:
-                    botlog(f'Successfully logged into {self.room}.', _copy = self._copy)
+                    log(f'Successfully logged into {self.room}.', _copy = self._copy)
                 else:
-                    botlog(f'Login unsuccessful. Reason: {reply.data.reason}', error = True, _copy = self._copy)
+                    log(f'Login unsuccessful. Reason: {reply.data.reason}', error = True, _copy = self._copy)
                     self._handle_auth(getpass("Enter the correct password: "))
                 break
             if i > self.APITimeout:
-                botlog("Auth Error: auth-reply API response not recorded.", error = True, _copy = self._copy)
+                log("Auth Error: auth-reply API response not recorded.", error = True, _copy = self._copy)
                 break
 
     def _handle_other(self, msg):
@@ -300,7 +340,7 @@ class Bot:
                 reply = msg
                 return reply.data.listing
             if i > self.APITimeout:
-                botlog("who-reply API response not recorded. reason: too many events before who-reply.", error = True, _copy = self._copy)
+                log("who-reply API response not recorded. reason: too many events before who-reply.", error = True, _copy = self._copy)
                 return None
 
     def move_to(self, roomName, password = "", _copyStart = False):
@@ -308,13 +348,13 @@ class Bot:
         self.password = password
         self.restart(_copyStart = _copyStart)
         if _copyStart:
-            botlog(f"Copy started in &{self.room}.", _copy = self._copy)
+            log(f"Copy started in &{self.room}.", _copy = self._copy)
 
     def set_handler(self, eventString, function):
         if callable(function):
             self.handlers[eventString] = function
         else:
-            botlog(f"WARNING: handler for {eventString} not callable, handler not set.", error = True, _copy = self._copy)
+            log(f"WARNING: handler for {eventString} not callable, handler not set.", error = True, _copy = self._copy)
 
     def set_handlers(self, handlers):
         for eventString, function in handlers.items():
@@ -365,14 +405,14 @@ class Bot:
             if msg.type == "login-reply":
                 reply = msg
                 if reply.data.success:
-                    botlog("Login successful.", _copy = self._copy)
+                    log("Login successful.", _copy = self._copy)
                     self.account = reply.data.account_id
                 else:
-                    botlog(f"Access denied. Reason: {reply.data.reason}", error = True, _copy = self._copy)
+                    log(f"Access denied. Reason: {reply.data.reason}", error = True, _copy = self._copy)
                     self.login(email = input("Enter your email: "), password = getpass("Enter your password: "), setDefaults = setDefaults)
                 return reply
             if i > self.APITimeout:
-                botlog("login-reply API response not recorded. reason: too many events before pm-initiate-reply.", error = True, _copy = self._copy)
+                log("login-reply API response not recorded. reason: too many events before pm-initiate-reply.", error = True, _copy = self._copy)
                 return None
 
     def initiate_pm(self, id, bot = None, room = "pmtest", hostEmail = None, hostPassword = None):
@@ -397,11 +437,11 @@ class Bot:
                     else:
                         self.copy_to("pm:{to.data.pm_id}")
                 else:
-                    botlog("Access Denied: This bot does not have permission to initiate PMs.", error = True, _copy = self._copy)
-                    botlog(f"reply: {msg}", error = True, _copy = self._copy)
+                    log("Access Denied: This bot does not have permission to initiate PMs.", error = True, _copy = self._copy)
+                    log(f"reply: {msg}", error = True, _copy = self._copy)
                 return to
             if i > self.APITimeout:
-                botlog("pm-initiate-reply API response not recorded. reason: too many events before pm-initiate-reply.", error = True, _copy = self._copy)
+                log("pm-initiate-reply API response not recorded. reason: too many events before pm-initiate-reply.", error = True, _copy = self._copy)
                 return None
 
 class BotError(Exception):
@@ -410,9 +450,9 @@ class BotError(Exception):
 class KilledError(BotError):
     def __init__(self, message = "bot killed."):
         self.message = message
-        botlog(f'KilledError: {message}', error = True)
+        log(f'KilledError: {message}', error = True)
 
 class BadMessageError(BotError):
     def __init__(self, message):
         self.message = message
-        botlog(f'BadMessageError: {message}', error = True)
+        log(f'BadMessageError: {message}', error = True)
